/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is Event Logger.
 *
 * The Initial Developer of the Original Code is
 *   Ben Basson <contact@cusser.net>
 * Portions created by the Initial Developer are Copyright (C) 2005
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *   Ben Basson <contact@cusser.net>
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */
 
var eventlogger =
{
  eventPrefs: null,
  appPrefs: null,
  windows: new Array(),
  eventDoc: null,
  eventList: null,

  init: function () 
  {
    eventlogger.initPrefs();
    
    // window._originalWindow defined in overlay.js
    eventlogger.windows.push(window._originalWindow);

    eventloggerPrefObserver.register();
    eventloggerWindowObserver.register();

    window.removeEventListener("load", eventlogger.init, false);
    window.addEventListener("unload", eventlogger.uninit, false);
    
    if (eventlogger.appPrefs.getBoolPref("logging"))
      eventlogger.startLogging();
  },
  
  initPrefs: function ()
  {
    var prefs = Components.classes['@mozilla.org/preferences-service;1']
                  .getService(Components.interfaces.nsIPrefService);
    eventlogger.eventPrefs = prefs.getBranch('eventlogger.events.');
    eventlogger.appPrefs = prefs.getBranch('eventlogger.');
  },

  uninit: function ()
  {
    for (var n in eventlogger.windows)
      eventlogger.removeEventListeners(eventlogger.windows[n]);
    
    eventloggerPrefObserver.unregister();
    eventloggerWindowObserver.unregister();
  },
  
  // starts logging, resumes if previous eventDump.xml is found and populated
  startLogging: function (aEvent)
  {
    var windowList = eventlogger.windows;
    for (var n in windowList)
      eventlogger.addEventListeners(windowList[n]);
 
    var eventDoc = eventloggerUtils.getDOMTree();
    if (!eventDoc)
      eventDoc = document.implementation.createDocument("", "", null);
    
    eventlogger.eventDoc = eventDoc;
    
    if (eventDoc.documentElement)
    {
      var eventList = eventlogger.eventList = eventDoc.documentElement;
    }
    else
    {
      var eventList = eventDoc.createElement("eventlist");
      eventlogger.eventList = eventDoc.appendChild(eventList);
    }
  },
  
  stopLogging: function ()
  {   
    var windowList = eventlogger.windows;
    for (var n in windowList)
      eventlogger.removeEventListeners(windowList[n]);
  },
  
  addEventListeners: function (aWindow)
  {
    var eventList = eventlogger.eventPrefs.getChildList("", {});
    for (var n in eventList)
      if (eventlogger.eventPrefs.getBoolPref(eventList[n]))
        eventlogger.addEventListener(aWindow, eventList[n]);
  },
  
  removeEventListeners: function (aWindow)
  {
    var eventList = eventlogger.eventPrefs.getChildList("", {});
    for (var n in eventList)
      eventlogger.removeEventListener(aWindow, eventList[n]);
  },
  
  addEventListener: function (aWindow, aEventType)
  {
    aWindow.addEventListener(aEventType, eventlogger.handleEvent, true);  
  },

  removeEventListener: function (aWindow, aEventType)
  {
    aWindow.removeEventListener(aEventType, eventlogger.handleEvent, true);
  },
  
  appendEvent: function (aEvent)
  {
    var newEvent = eventlogger.eventDoc.createElement("event");
    for (var n in aEvent)
    {
      if (aEvent[n] && !(aEvent[n] instanceof Function || n.toUpperCase() == n))
      {
        if (aEvent[n].toString().toLowerCase().indexOf("window") != -1)
        {
          var windowAttribs = eventlogger.getUniqueWindow(aEvent[n]);
          newEvent.setAttribute(n, windowAttribs);
        }
        else if (aEvent[n].toString().indexOf("object") != -1)
        {
          var objAttribs = eventlogger.getUniqueIdentifier(aEvent[n]);
          newEvent.setAttribute(n, objAttribs);
        }
        else
          newEvent.setAttribute(n, aEvent[n]);
      }
    }
    
    eventlogger.eventList.appendChild(newEvent);
    eventloggerUtils.outputEventList(eventlogger.eventDoc);
    
    var serializer = new XMLSerializer();
    var eventXML = serializer.serializeToString(newEvent);
    eventloggerUtils.notifyObservers(eventXML);
  },
  
  handleEvent: function (aEvent)
  {
    eventlogger.appendEvent(aEvent);
  },

  /*
    XXX areas for improvement:
    1) should check uniqueness 
    2) needs to cover more attributes
  */
  
  getUniqueIdentifier: function (aTarget)
  {
    // this avoids returning pseudo-elements
    while (aTarget.prefix)
      aTarget = aTarget.parentNode;
    
    if (aTarget.id != null && aTarget.id != "")
      return ["id", aTarget.id];
    
    if (aTarget.label != null & aTarget.label != "")
      return ["label", aTarget.label];
    
    if (aTarget.title != null && aTarget.title != "")
      return ["title", aTarget.title];
        
    if (aTarget.tooltipText != null && aTarget.tooltipText != "")
      return ["tooltipText", aTarget.tooltipText];

    if (aTarget.tagName)
    {
      var tagName = aTarget.tagName;
      var tags = aTarget.ownerDocument.getElementsByTagName(tagName);

      for (var index in tags)
      {
        if (tags[index] == aTarget)
          return ["tagName", tagName, index];
      }
    }
    
    dump("getUniqueIdentifier failed for " + aTarget.localName + "\n");
    return false;
  },
  
  /*
    XXX areas for improvement:
    1) probably needs a cleaner way of returning the values
    2) needs to differentiate non-chrome windows somehow,
       is currently ignoring them
  */
  
  getUniqueWindow: function (aTarget)
  {
    var unique = true;
    var index = 0;
    var windows = eventlogger.windows;
    var targetId = aTarget.id;
    
    for (var n in windows)
    {
      if (aTarget.toString() == "[object ChromeWindow]")
      {
        if (windows[n] == aTarget)
          index = n;
        if (windows[n] != aTarget && windows[n].id == targetId)
          unique = false;
      }
    }

    if (!unique)
      return [targetId, index]
    else
      return targetId;
  },

};

var eventloggerPrefObserver =
{
  register: function()
  {
    if (!eventlogger.eventPrefs || !eventlogger.appPrefs)
      return;

    var appPrefs = eventlogger.appPrefs.QueryInterface(Components.interfaces.nsIPrefBranch2);
    appPrefs.addObserver("", this, false);
    
    var eventPrefs = eventlogger.eventPrefs.QueryInterface(Components.interfaces.nsIPrefBranch2);
    eventPrefs.addObserver("", this, false);
  },

  unregister: function()
  {
    var appPrefs = eventlogger.appPrefs.QueryInterface(Components.interfaces.nsIPrefBranch2);
    appPrefs.removeObserver("", this, false);
    
    var eventPrefs = eventlogger.eventPrefs.QueryInterface(Components.interfaces.nsIPrefBranch2);
    eventPrefs.removeObserver("", this, false);
  },

  observe: function(aSubject, aTopic, aData)
  {
    if (aTopic != "nsPref:changed")
      return;
      
    if (aData == "logging")
    {
      if (eventlogger.appPrefs.getBoolPref(aData))
        eventlogger.startLogging();
      else
        eventlogger.stopLogging();
      return;
    }
    
    aData = aData.substring(7, aData.length);
    var addEvent = eventlogger.eventPrefs.getBoolPref(aData)
    for (var n in eventlogger.windows)
    {
      if (addEvent)
        eventlogger.addEventListener(eventlogger.windows[n], aData);
      else
        eventlogger.removeEventListener(eventlogger.windows[n], aData);
    }
  }
};

var eventloggerWindowObserver =
{
  register: function()
  {
    var os = Components.classes["@mozilla.org/observer-service;1"]
               .getService(Components.interfaces.nsIObserverService);
               
    os.addObserver(this, "domwindowopened", false);
    os.addObserver(this, "domwindowclosed", false);
  },

  unregister: function ()
  {
    var os = Components.classes["@mozilla.org/observer-service;1"]
               .getService(Components.interfaces.nsIObserverService);
               
    os.removeObserver(this, "domwindowopened", false);
    os.removeObserver(this, "domwindowclosed", false);
  },
  
  observe: function (aSubject, aTopic, aData)
  {
    if (aTopic == "domwindowopened")
    {
      eventlogger.windows.push(aSubject);
      if (eventlogger.appPrefs.getBoolPref("logging"))
        eventlogger.addEventListeners(aSubject);
    }
    else
    {
      var index = eventlogger.windows.indexOf(aSubject);
      eventlogger.windows.splice(index,1);
    }
  }
};

window.addEventListener("load", eventlogger.init, false);