/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is Event Logger.
 *
 * The Initial Developer of the Original Code is
 *   Ben Basson <contact@cusser.net>
 * Portions created by the Initial Developer are Copyright (C) 2005
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *   Ben Basson <contact@cusser.net>
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */
 
var eventloggerWindow = 
{
  appPrefs: null,
  listBox: null,
  
  init: function ()
  {
    var prefs = Components.classes['@mozilla.org/preferences-service;1']
                  .getService(Components.interfaces.nsIPrefService);
    var appPrefs = eventloggerWindow.appPrefs = prefs.getBranch('eventlogger.');
    
    eventloggerUpdateObserver.register();

    eventloggerWindow.setButtons(appPrefs.getBoolPref("logging"));
    window.removeEventListener("load", eventloggerWindow.init, false);
    window.addEventListener("unload", eventloggerWindow.uninit, false);

    eventloggerWindow.listBox = document.getElementById("event-list");
    eventloggerWindow.populateListBox();
  },

  uninit: function ()
  {
    eventloggerUpdateObserver.unregister();  
  },
  
  startLogging: function ()
  {
    eventloggerWindow.appPrefs.setBoolPref("logging",true);
    eventloggerWindow.setButtons(true);
  },
  
  stopLogging: function ()
  {
    eventloggerWindow.appPrefs.setBoolPref("logging",false);
    eventloggerWindow.setButtons(false);
  },
  
  resetLog: function ()
  {
    var file = eventloggerUtils.getTempFile();
    if (file.exists())
      file.remove(false);
    eventloggerWindow.resetListBox();
  },
  
  setButtons: function (aBool)
  {
    document.getElementById("startLogging").setAttribute("disabled",aBool);
    document.getElementById("stopLogging").setAttribute("disabled",!aBool);
    document.getElementById("resetLog").setAttribute("disabled",aBool);
  },
  
  populateListBox: function ()
  {
    var eventDoc = eventloggerUtils.getDOMTree();
    if (eventDoc && eventDoc.documentElement)
    {
      var eventList = eventDoc.documentElement.childNodes;
      if (eventList.length && eventList.length > 0)
        for (var n = 0; n < eventList.length; n++)
          eventloggerWindow.addEvent(eventList[n]);
    }
  },
  
  addEvent: function (aEventNode)
  {
    if (!aEventNode)
      return;
    
    const xulxmlns = "http://www.mozilla.org/keymaster/gatekeeper/there.is.only.xul";
    var listBox = eventloggerWindow.listBox;
    
    var newEvent = document.createElementNS(xulxmlns, "listitem");
    var newEventLabel = aEventNode.getAttribute("type") + " at " + aEventNode.getAttribute("originalTarget");
    newEvent.setAttribute("label", newEventLabel);
    
    var rows = listBox.getRowCount();
    listBox.appendChild(newEvent);
    listBox.ensureIndexIsVisible(rows);
  },

  resetListBox: function ()
  {
    var listBox = eventloggerWindow.listBox;
    for (var n = listBox.childNodes.length - 1; n >= 0; n--)
      listBox.removeChild(listBox.childNodes[n]);
  }
};

var eventloggerUpdateObserver =
{
  register: function()
  {
    var os = Components.classes["@mozilla.org/observer-service;1"]
               .getService(Components.interfaces.nsIObserverService);
               
    os.addObserver(this, "eventLoggerUpdate", false);
  },

  unregister: function ()
  {
    var os = Components.classes["@mozilla.org/observer-service;1"]
               .getService(Components.interfaces.nsIObserverService);
               
    os.removeObserver(this, "eventLoggerUpdate", false);
  },
  
  observe: function (aSubject, aTopic, aData)
  {
    if (aData)
    {
      var parser = new DOMParser();
      var eventNode = parser.parseFromString(aData, "text/xml").firstChild;
      eventloggerWindow.addEvent(eventNode);
    }
  }
};

window.addEventListener("load", eventloggerWindow.init, false);